;****************************************************
;lab4
;approx 2 s delay program with 2 status leds to show
;start and end of delay.
;will use tmr0 peripheral w/ interrupts
;****************************************************

	List p=16f84A
	include "P16f84A.inc"		;load the include file
	__CONFIG  _XT_OSC & _WDT_OFF & _CP_OFF & _PWRTE_ON

;declare registers used
		cblock	0x0C
COUNTER
W_TEMP
STATUS_TEMP
		endc

;COUNTER	equ	0x0C

;=====================================================================

		org	0x00		;indicate the start location
	goto	Start

;interrupt vector
		org	0x04
	goto	Int_serv_routine


;program memory here
		org	0x05
;=====================================================================

;call routines

;routine to preload tmr0
Settmr0
	movlw	0x00			;reset tmr0 to count again
	movwf	TMR0			;load tmr0
	return

;routine to preload COUNTER
Setcounter
	movlw	0x1E			;setup counter with countdown value that gives
	movwf	COUNTER			;2 sec delay
	return

;routine to decrement and test COUNTER
Decr_counter
	decf	COUNTER,F		;decrement counter by 1
	btfsc	STATUS,Z		;test to see if (1Eh) rounds of tmr0 have been run
	call	Light_2nd_led		;
	call	Settmr0
	return

;routine to light 2nd led
Light_2nd_led
	bsf	PORTB,3			;light 2nd led to tell when time expired
	return

;====================================================================

;setup tmr0 and interrupts
Start
	movlw	b'01000111'		;setup OPTION register
	bsf	STATUS,RP0		;setup bank pointer to access OPTION register
	movwf	OPTION_REG		;move value into option
	bcf	STATUS,RP0		;clear bank pointer for normal operation
	movlw	b'10100000'		;setup INTCON register
	movwf	INTCON			;move value into INTCON

;reset portb/initialize routine
Portb
					;clear PORTB
	clrf	PORTB			;load output latch w/zeros
	bsf	STATUS,RP0		;setup bank pointer to access TRIS register
	clrf	TRISB			;configure ports to outputs
	bcf	STATUS,RP0		;clear bank pointer for normal operation

	call	Settmr0
	call	Setcounter

;====================================================================
Main

;light 1st led to tell user code has started delay counter
	bsf	PORTB,2			;light the LSB (RB0) on PORTB
Delay
					;wait here til tmr0 interrupt
	goto	$			;$ = current address... infinite loop


;====================================================================

Int_serv_routine
;push W and STATUS
	movwf	W_TEMP			;push W and STATUS
	swapf	STATUS,W		;
	movwf	STATUS_TEMP		;

	
;check to see what caused the interrupt
					;test for tmr0 causing interrupt
	btfsc	INTCON,T0IF		;test tmr0 flag
	call	Decr_counter		;

	swapf	STATUS_TEMP,W		;pop W and STATUS
	movwf	STATUS			;
	swapf	W_TEMP,F		;
	swapf	W_TEMP,W		;
	bcf	INTCON,T0IF		;clear tmr0 flag
	retfie				;return back since not tmr0 flag


	


	end

