;****************************************************
;lab5
;use approx 2 s flash/delay program to control 2 door motors 
;w/operating cycle period of 2 sec. 
;will use tmr0 peripheral w/ interrupts and porta
;****************************************************

	List p=16f84A
	include "P16f84A.inc"		;load the include file
	__CONFIG  _XT_OSC & _WDT_OFF & _CP_OFF & _PWRTE_ON

;declare registers used
		cblock	0x0C
COUNTER
STATBIT
W_TEMP
STATUS_TEMP
		endc

;COUNTER	equ	0x0C

;=====================================================================

		org	0x00		;indicate the start location
	goto	Start

;interrupt vector
		org	0x04
	goto	Int_serv_routine


		org	0x05
;====================================================================
;call routines

;routine to preload tmr0
Settmr0
	movlw	0x00			;reset tmr0 to count again
	movwf	TMR0			;load tmr0
	return

;routine to preload COUNTER
Setcounter
	movlw	0x1E			;setup counter with countdown value that gives
	movwf	COUNTER			;2 sec delay
	return

;routine to decrement and test COUNTER
Decr_counter
	decf	COUNTER,F		;decrement counter by 1
	btfsc	STATUS,Z		;test to see if (1Eh) rounds of tmr0 have been run
	call	Open_close_door		;
	call	Settmr0
	return

;motor control logic
Open_close_door
					;test status bit to see which direction to drive
	movlw	0x01			;load W with value to xor
	xorwf	STATBIT,F		;bit toggle status bit for repeating
	btfsc	STATUS,Z		;test to see if open phase or close phase
	goto	Open_phase		;
	goto	Close_phase		;

;open doors
Open_phase
					;turn motors on in open direction
					;must stop previous direction 1st
	clrf	PORTA			;stop previous movement
	movlw	b'00000110'		;load W with value to xor
	xorwf	PORTA,F			;bit toggle control bits 
	call	Setcounter
	return

;close doors
Close_phase
					;turn motors on in close direction
					;must stop previous direction 1st
	clrf	PORTA			;stop previous movement
	movlw	b'00001001'		;load W with value to xor
	xorwf	PORTA,F			;bit toggle control bits 
	call	Setcounter
	return

;setup stat bit
Setstatbit
	movlw	0x01			;preload value for statbit
	movwf	STATBIT			;
;	bsf	STATBIT,0		;setup status bit
	return

;====================================================================

;setup tmr0 and interrupts
Start
	movlw	b'01000111'		;setup OPTION register
	bsf	STATUS,RP0		;setup bank pointer to access OPTION register
	movwf	OPTION_REG		;move value into option
	bcf	STATUS,RP0		;clear bank pointer for normal operation
	movlw	b'11100000'		;setup INTCON register
	movwf	INTCON			;move value into INTCON

;reset porta/initialize routine
Porta
					;clear PORTA
	clrf	PORTA			;load output latch w/zeros
	bsf	STATUS,RP0		;setup bank pointer to access TRIS register
	clrf	TRISA			;configure ports to outputs
	bcf	STATUS,RP0		;clear bank pointer for normal operation


;setup stat bit
	call	Setstatbit		;	

;====================================================================

Main

;light led to tell user code has started delay counter
;	bsf	PORTB,0			;light the LSB (RB0) on PORTB

	call	Settmr0
	call	Setcounter

Delay

	goto	$			;$ = current address... infinite loop




;======================================================================

Int_serv_routine
	movwf	W_TEMP			;push W and STATUS
	swapf	STATUS,W		;
	movwf	STATUS_TEMP		;

;check to see what caused the interrupt
					;test for tmr0 causing interrupt
	btfsc	INTCON,T0IF		;test tmr0 flag
	call	Decr_counter		;

	swapf	STATUS_TEMP,W		;pop W and STATUS
	movwf	STATUS			;
	swapf	W_TEMP,F		;
	swapf	W_TEMP,W		;
	bcf	INTCON,T0IF		;clear tmr0 flag
	retfie





	end

