;****************************************************
;lab6a
;use polling of an input pin to start door control sequence 
;will use tmr0 peripheral w/ interrupts and porta
;will read RB1 push button as active lo input 
;****************************************************

	List p=16f84A
	include "P16f84A.inc"		;load the include file
	__CONFIG  _XT_OSC & _WDT_OFF & _CP_OFF & _PWRTE_ON

;declare registers used
		cblock	0x0C
COUNTER
STATBIT
DEB_COUNTER
W_TEMP
STATUS_TEMP
		endc

;COUNTER	equ	0x0C

;=====================================================================

		org	0x00		;indicate the start location
	goto	Start

;interrupt vector
		org	0x04
	goto	Int_serv_routine


		org	0x05
;====================================================================
;call routines

;routine to check for key input & signal bounce
Check_key
					;poll for key press
	btfsc	PORTB,1			;
	goto	Check_key		;
	call	Debounce_delay		;debounce signal
	btfsc	PORTB,1			;check for bouncing input
	goto	Check_key		;
	return

;routine to give 256us delay for key bounce routine
Debounce_delay
	movlw	0xFF			;load debounce delay
	movwf	DEB_COUNTER		;
Loop_debounce
	decfsz	DEB_COUNTER,F		;loop to give 256us settle time
	goto	Loop_debounce		;
	return

;routine to preload tmr0
Settmr0
	movlw	0x00			;reset tmr0 to count again
	movwf	TMR0			;load tmr0
	return

;routine to preload COUNTER
Setcounter
	movlw	0x1E			;setup counter with countdown value that gives
	movwf	COUNTER			;2 sec delay
	return

;routine to preload STATBIT
Setstatbit
	movlw	0x04			;setup 4 state cycle counter
	movwf	STATBIT			;load status bit
	return

;routine to decrement and test COUNTER
Decr_counter
	decf	COUNTER,F		;decrement counter by 1
	btfsc	STATUS,Z		;test to see if 76x (4Ch) rounds of tmr0 have been run
	call	Open_close_door		;
	call	Settmr0
	return

;motor control logic
Open_close_door
					;test status bit to see which direction to drive
	decf	STATBIT,F		;decr state machine
	movlw	0x03			;load W with state to test against
	subwf	STATBIT,W		;sub to test state 3
	btfsc	STATUS,Z		;test to see what phase
	goto	Open_phase		;
	movlw	0x02			;load W with state to test against
	subwf	STATBIT,W		;sub to test state 2	
	btfsc	STATUS,Z		;test to see what phase
	goto	Close_phase		;
	movlw	0x01			;load W with state to test against
	subwf	STATBIT,W		;sub to test state 1
	btfsc	STATUS,Z		;test to see what phase
	goto	End_phase		;

;open doors
Open_phase
					;turn motors on in open direction
					;must stop previous direction 1st
	clrf	PORTA			;stop previous movement
	movlw	b'00000110'		;load W with value to xor
	xorwf	PORTA,F			;bit toggle control bits 
	call	Setcounter
	return

;close doors
Close_phase
					;turn motors on in close direction
					;must stop previous direction 1st
	clrf	PORTA			;stop previous movement
	movlw	b'00001001'		;load W with value to xor
	xorwf	PORTA,F			;bit toggle control bits 
	call	Setcounter
	return

;stop motors/end of cycle
End_phase
					;stop motors in close position
					;must stop previous direction 1st
	clrf	PORTA			;stop previous movement
	call	Setcounter
	call	Setstatbit
	call	Check_key		;check for next key press
	return

;====================================================================

Start
	movlw	b'11000111'		;setup OPTION register
	bsf	STATUS,RP0		;setup bank pointer to access OPTION register
	movwf	OPTION_REG		;move value into option
	bcf	STATUS,RP0		;clear bank pointer for normal operation
	movlw	b'00100000'		;setup INTCON register
	movwf	INTCON			;move value into INTCON

;reset ports/initialize routine
Porta
					;clear PORTA
	clrf	PORTA			;load output latch w/zeros
	bsf	STATUS,RP0		;setup bank pointer to access TRIS register
	clrf	TRISA			;configure ports to outputs
	bcf	STATUS,RP0		;clear bank pointer for normal operation

Portb
					;clear PORTB
	clrf	PORTB			;load output latch w/zeros
	movlw	0xFF			;load FF into W
	bsf	STATUS,RP0		;setup bank pointer to access TRIS register
	movwf	TRISB			;configure ports to inputs
	bcf	STATUS,RP0		;clear bank pointer for normal operation


	

;====================================================================

MAIN

;light led to tell user code has started delay counter
;	bsf	PORTB,0			;light the LSB (RB0) on PORTB
	call	Check_key
	call	Setstatbit
	call	Settmr0
	call	Setcounter		

	bsf	INTCON,GIE		;turn on interrupts now that key detected
DELAY
COUNTDOWN
	goto	$			;$ = current address... infinite loop




;======================================================================

Int_serv_routine
	movwf	W_TEMP			;push W and STATUS
	swapf	STATUS,W		;
	movwf	STATUS_TEMP		;

;check to see what caused the interrupt
					;test for tmr0 causing interrupt
	btfsc	INTCON,T0IF		;test tmr0 flag
	call	Decr_counter		;

	swapf	STATUS_TEMP,W		;pop W and STATUS
	movwf	STATUS			;
	swapf	W_TEMP,F		;
	swapf	W_TEMP,W		;
	bcf	INTCON,T0IF		;clear tmr0 flag
	retfie




	end

