;****************************************************
;lab8
;use interrupt on INT pin to start door control sequence 
;will use tmr0 peripheral 
;will read RB0 opto switch active lo input
;will hold door open for 2 sec before closing
;will use motor cutoff switches
;will zero doors prior to running any other code 
;will indicate if door jammed in full open/closed position
;will not close door if opto still active (people still in doorway)
;****************************************************
	List p=16f84A
	include "P16f84A.inc"		;load the include file
	__CONFIG  _XT_OSC & _WDT_OFF & _CP_OFF & _PWRTE_OFF

;declare registers used
		cblock	0x0C
COUNTER
STATBIT
READ
DELAYCOUNT
DELAYCOUNT1
DELAYCOUNT2
DEBOUNCE
W_TEMP
STATUS_TEMP
		endc

;COUNTER	equ	0x0C

;=====================================================================

		org	0x00		;indicate the start location
	goto	Start

;interrupt vector
		org	0x04
	goto	Int_serv_routine


		org	0x05
;=====================================================================
;call routines

;routine to preload tmr0
Settmr0
	movlw	0x00			;reset tmr0 to count again
	movwf	TMR0			;load tmr0
	return

;routine to preload COUNTER
Setcounter
	movlw	0x1E			;setup counter with countdown value that gives
	movwf	COUNTER			;5 sec delay
	return

;routine to preload STATBIT
Setstatbit
	movlw	0x03			;setup 4 state cycle counter
	movwf	STATBIT			;load status bit
	return

;routine to setup interrupts for tmr0
Tmr0ints
	bcf	INTCON,GIE		;disable ints while configing
	bcf	INTCON,INTF		;must clear int flag 1st
	bcf	INTCON,INTE		;disable int IE
	bcf	INTCON,RBIF		;must clear woc flag 1st
	bcf	INTCON,RBIE		;disable woc IE
	bcf	INTCON,T0IF		;must clear tmr0 flag 1st
;	bsf	INTCON,T0IE		;enable tmr0 IE
	bsf	INTCON,GIE		;turn on interrupts since serv routine
	return

;routine to setup interrupts for INT pin
Intints
	bcf	INTCON,GIE		;disable ints while configing
	bcf	INTCON,T0IF		;must clear tmr0 flag 1st
	bcf	INTCON,T0IE		;disable tmr0 IE
	bcf	INTCON,RBIF		;must clear woc flag 1st
	bcf	INTCON,RBIE		;disable woc IE
	bcf	INTCON,INTF		;must clear int flag 1st
	bsf	INTCON,INTE		;enable int IE
	bsf	INTCON,GIE		;turn on interrupts since serv routine
	return

;routine to setup interrupts for woc
Wocints
	bcf	INTCON,GIE		;disable ints while configing
	bcf	INTCON,T0IF		;must clear tmr0 flag 1st
	bcf	INTCON,T0IE		;disable tmr0 IE
	bcf	INTCON,INTF		;must clear int flag 1st
	bcf	INTCON,INTE		;disable int IE
	bcf	INTCON,RBIF		;must clear woc flag 1st
	movf	PORTB,F			;clear mismatch condition
	bsf	INTCON,RBIE		;enable woc IE
	bsf	INTCON,GIE		;turn on interrupts since serv routine
	return
		
;routine to decrement and test COUNTER
Decr_counter
	decf	COUNTER,F		;decrement counter by 1
	btfsc	STATUS,Z		;test to see if (1Eh) rounds of tmr0 have been run
	call	Open_close_door		;
	return

;motor control logic
Open_close_door
					;test status bit to see which direction to drive
	decf	STATBIT,F		;decr state machine
	movlw	0x02			;load W with state to test against
	subwf	STATBIT,W		;sub to test state 2
	btfsc	STATUS,Z		;test to see what phase
	goto	Open_phase		;
	movlw	0x01			;load W with state to test against
	subwf	STATBIT,W		;sub to test state 1
	btfsc	STATUS,Z		;test to see what phase
	goto	Close_phase		;


;open doors
Open_phase
					;turn motors on in open direction
					;must stop previous direction 1st
	clrf	PORTA			;stop previous movement
	movlw	b'00000110'		;load W with value to xor
	xorwf	PORTA,F			;bit toggle control bits 
	call	Delay_switch		;delay to give motors time to clear existing sw's
	movf	PORTB,W			;check to see if off the existing switches yet
	movwf	READ			;doors must clear switches they are on before they can
					;trigger the cuttoffs

	movlw	b'01010000'		;mask off unwanted bits
	andwf	READ,F			;
	subwf	READ,F			;test to see if both switches cleared yet
	btfss	STATUS,Z		;
	call	Jammed			;door jammed

	call	Wocints			;setup woc ints to stop motors
	sleep
	nop
	bcf	INTCON,GIE		;no more interrupts here
Check_open
					;read portB pins and mask to test
	movf	PORTB,W			;load read value into temp register
	movwf	READ			;
	movlw	b'10100000'		;mask out motor closed sensors and unwanted bits
	andwf	READ,F			;test to see if both motors open
	btfss	STATUS,Z		
	goto	Check_open		;loop back since only 1 motor open so far
	clrf	PORTA			;stop all motors
	call	Tmr0ints		;setup tmr0 ints
					;pause_phase built into return point of loop
	call	Setcounter		;this takes the place of pause phase 
	call	Settmr0			;since returning forces another tmr0 countdown
	return				;must preload tmr0 since loading is at end of holding loop


;close doors
Close_phase
					;check to see if people clear of door first
					;for close cycle only
	btfss	PORTB,0			;see if someone still blocking opto
	goto	Close_phase		;


					;turn motors on in close direction
					;must stop previous direction 1st
	clrf	PORTA			;stop previous movement
	movlw	b'00001001'		;load W with value to xor
	xorwf	PORTA,F			;bit toggle control bits 
	call	Delay_switch
	movf	PORTB,W			;check to see if off the existing switches yet
	movwf	READ			;doors must clear switches they are on before they can
					;trigger the cuttoffs
	
	movlw	b'10100000'		;mask off unwanted bits
	andwf	READ,F			;
	subwf	READ,F			;test to see if both switches cleared yet
	btfss	STATUS,Z		;
	call	Jammed			;

	call	Wocints			;setup woc ints to stop motors
	sleep
	nop
	bcf	INTCON,GIE		;no more interrupts here
Check_closed
					;read portB pins and mask to test
	movf	PORTB,W			;load read value into temp register
	movwf	READ			;
	movlw	b'01010000'		;mask out motor open sensors and unwanted bits
	andwf	READ,F			;test to see if both motors closed
	btfss	STATUS,Z		
	goto	Check_closed		;loop back since only 1 motor closed so far
	clrf	PORTA			;stop all motors
					;end_phase built into end of close_phase
;stop motors/end of cycle
End_phase
					;stop motors in close position
					;must stop previous direction 1st
	clrf	PORTA			;stop previous movement
	movlw	0x01			;must preload counter with smallest value
	movwf	COUNTER			;so that delay btwn opto to start motors is small 
	call	Setstatbit
	call	Intints
	sleep
	nop
	call	Tmr0ints
	movlw	0xFE			;must preload tmr0 with smallest value
	movwf	TMR0			;so that delay btwn opto to start motors is small
	return


;check which one of the sensors is giving feed back
Feed_back
	call	Delay_debounce		;debounce position feedback
					;read portB woc pins and mask to test
	movf	PORTB,W			;load read value into temp register
	movwf	READ			;
	movlw	b'11110000'		;mask out unwanted bits so can use READ elsewhere
	andwf	READ,F			;
	btfss	READ,7			;check motor2 open
	call	Kill_motor2		;
	btfss	READ,6			;check motor2 closed
	call	Kill_motor2		;
	btfss	READ,5			;check motor1 open
	call	Kill_motor1		;
	btfss	READ,4			;check motor1 closed
	call	Kill_motor1		;
	return

;stop of motor2
Kill_motor2
					;stop motion on motor2
	movlw	b'00000011'		;load stop value (will not stop motor1)
	andwf	PORTA,F			;stop the motor
;	bcf	PORTA,3
;	bcf	PORTA,2
	return

;stop of motor1
Kill_motor1
					;stop motion on motor1
	movlw	b'00001100'		;load stop value (will not stop motor2)
	andwf	PORTA,F			;stop the motor
;	bcf	PORTA,1
;	bcf	PORTA,0
;	bsf	PORTB,2
;	goto	$

	return
	 
;routine to give motors time to clear off of the exsisting switch
Delay_switch
	movlw	0x04
	movwf	DELAYCOUNT2
Loop2
	movlw	0xFA
	movwf	DELAYCOUNT1
Loop1
	movlw	0xFA
	movwf	DELAYCOUNT
Loop
	decfsz	DELAYCOUNT,F
	goto	Loop
	decfsz	DELAYCOUNT1,F	
	goto	Loop1
	decfsz	DELAYCOUNT2,F
	goto	Loop2
	return

;door jammed routine
Jammed
	clrf	PORTA			;emergency stop of doors
	bsf	PORTB,2			;light door jammed led
	goto	$			;crash until reset switch is hit

;routine to delay till tmr0 rolls over
;part of tmr0 timing routine since not
;using tmr0 interrupt but just the flag
Delay
					;will only delay for 1 tmr0 roll over
	btfss	INTCON,T0IF		;check tmr0 flag to see if rolled over
	goto	Delay
	bcf	INTCON,T0IF		;done checking flag
	return

;delay for debounce routine
Delay_debounce
	movlw	0xFF			;setup 256us delay
	movwf	DEBOUNCE		;
Debounce_loop
	decfsz	DEBOUNCE,F		;count down the debounce counter
	goto	Debounce_loop		;
	return
;=====================================================================

Start
	movlw	b'10000111'		;setup OPTION register
	bsf	STATUS,RP0		;setup bank pointer to access OPTION register
	movwf	OPTION_REG		;move value into option
	bcf	STATUS,RP0		;clear bank pointer for normal operation


;=========================
;reset ports/initialize routine
Porta
					;clear PORTA
	clrf	PORTA			;load output latch w/zeros
	bsf	STATUS,RP0		;setup bank pointer to access TRIS register
	clrf	TRISA			;configure ports to outputs
	bcf	STATUS,RP0		;clear bank pointer for normal operation

Portb
					;clear PORTB
	clrf	PORTB			;load output latch w/zeros
	movlw	b'11111011'		;RB2 is output for door jammed
	bsf	STATUS,RP0		;setup bank pointer to access TRIS register
	movwf	TRISB			;configure ports to inputs
	bcf	STATUS,RP0		;clear bank pointer for normal operation



;====================================================================

MAIN

;light led to tell user code has started delay counter
;	bsf	PORTB,0			;light the LSB (RB0) on PORTB
	call	Close_phase		;zero doors
;	call	Setcounter
;	call	Setstatbit

;holding loop to wait for tmr0 to roll over the needed
;number of times
Hold_here
	call	Delay
	call	Decr_counter		
	call	Settmr0
	goto	Hold_here





;======================================================================

Int_serv_routine
	movwf	W_TEMP			;push W and STATUS
	swapf	STATUS,W		;
	movwf	STATUS_TEMP		;

;check to see what caused the interrupt
RB0_INT
					;test for RB0/int causing interrupt
	btfss	INTCON,INTF		;test int flag
	goto	Woc_int			;
	bcf	INTCON,INTF		;clear INTF so can go on and open/close
	swapf	STATUS_TEMP,W		;pop W and STATUS
	movwf	STATUS			;
	swapf	W_TEMP,F		;
	swapf	W_TEMP,W		;
	retfie

Woc_int

					;test for woc causing interrupt
	btfsc	INTCON,RBIF		;test woc flag
	call	Feed_back		;
	bcf	INTCON,RBIF		;clear woc flag
	swapf	STATUS_TEMP,W		;pop W and STATUS
	movwf	STATUS			;
	swapf	W_TEMP,F		;
	swapf	W_TEMP,W		;
	retfie
		
;this routine not used in this program
;since not using tmr0 interrupt, just tmr0 flag
;Tmr0_int
					;test for tmr0 causing interrupt
;	btfsc	INTCON,T0IF		;test tmr0 flag
;	call	Decr_counter		;
;	bcf	INTCON,T0IF		;clear tmr0 flag
;	call	Settmr0			;
;	swapf	STATUS_TEMP,W		;pop W and STATUS
;	movwf	STATUS			;
;	swapf	W_TEMP,F		;
;	swapf	W_TEMP,W		;
;	retfie



	end

